import socketserver
import logging
import requests
import json
from datetime import datetime
from cshmac.requests import HmacAuth

LOG_FILE_PATH = '/var/log/cyops/cyops-integrations/syslog/listener.log'

logging.basicConfig(filename=LOG_FILE_PATH,level=logging.DEBUG,
                    format='%(asctime)s %(levelname)s %(module)s %(funcName)s(): %(message)s')
cs_host = 'localhost'
DEFAULT_ALGORITHM = 'sha256'
timestamp = datetime.utcnow().strftime("%Y-%m-%d %H:%M:%S")

logger = logging.getLogger(__name__)
SSL_VERIFY = False

class SyslogHandler(socketserver.BaseRequestHandler):

    def handle(self):
        logger.info('got new message')
        if self.server.transport and self.server.transport.lower() == 'tcp':
            data = self.request.recv(1024).decode('utf-8')
        else:
            #assuming udp by default
            data = bytes.decode(self.request[0].strip())
        logger.info("%s : %s" % (self.client_address[0], str(data)))
        logger.info('filter string is :%s' % self.server.filter_str)
        if (not self.server.filter_str) or (self.server.filter_str.strip() == '') or (self.server.filter_str in data):
            logger.info('Triggering playbook for message.')
            self.forward_to_cyops(data)
        else:
            logger.info('Message did not match the filter string. Not triggering playbook.')

    def forward_to_cyops(self, data):
        if data:
            # Replace if any unicode sequences in data
            data = data.replace("\u0000",'')
            full_uri = 'https://' + cs_host + '/api/triggers/1/' + self.server.trigger
            try:
                auth = HmacAuth(full_uri, 'post', self.server.appliance_public_key, self.server.appliance_private_key, data)
                response = requests.request('post', full_uri, auth=auth, data=data, verify=SSL_VERIFY)
                if response.ok:
                    logger.info('Playbook for fetching mail triggered successfully')
                else:
                    response.raise_for_status()
            except Exception as e:
                logging.exception(
                    "Error occurred while triggering the CyOPs playbook ' {0} ' ERROR :: {1}".format(self.server.trigger, e))
                raise Exception(
                    "Error occurred while triggering the CyOPs playbook ' {0} ' ERROR :: {1}".format(self.server.trigger, e))

